/**
 * Simple class containing examples of for and while loops.
 */
public class Looper {
    /**
     * Constructor for objects of class Looper sets the 
     * instance variables to default values.
     */
    public Looper() {
        base = 100.0;        // initial amount
        rate = 0.1;          // 10 % growth per period
    }

    /**
     * simple "for" loop demo using println
     * @param  count how many loops to perform
     */
    public void demoFor(int count) {
        /*
         * "i" is a local variable, created fresh each
         * time demoFor is called.  It needs to be initialized
         * before use.
         */
        int i;      
       /* 
         * Format of the "for" statement is:
         *       for (initialize; test for run condition; update) {body}
         * The "initialize" is done once, before the body executes at all.
         * The "test for run condition" is done before the body executes
         * each loop.  If true, the body is executed.  If false, the "for"
         * statement is complete and so the next statement after the body
         * is executed. The "update" is done each time after the body of 
         * the loop executes. It should be doing something that is getting
         * us closer to finishing. The update in a for loop is usually very
         * simple, like incrementing the loop counter variable.
         */
        System.out.println("\nCount up : count = "+count);
        for (i=0; i < count; i++) {                         // 0-based, up to count-1
            System.out.println("i : "+i);
        }
        System.out.println("i is now : "+i);
        
        System.out.println("\nCount up : count = "+count);
        for (i=1; i <= count; i++) {                        // 1-based, up to count
            System.out.println("i : "+i);
        }
        
        System.out.println("\nCount down : count = "+count);
        for (i = count-1; i >= 0; i--) {                    // down to 0 from count-1
            System.out.println("i : "+i);
        }
        System.out.println("\nCount down : count = "+count);
        for (int j = count; j > 0; j--) {                   // down to 1 from count
            System.out.println("j : "+j);
        }
//        System.out.println("j is now " + j);          // can't do it, j not defined here

    }
    /**
     * simple "while" loop demo using println and implementing a growth
     * calculation.
     * @param  goal the number that we want to reach, starting from
     * the base amount and growing at rate amount per period
     */
    public void demoWhile(double goal) {
    
        int periods;        // some local variables for later use
        double toDate;      // note that they are uninitialized at the moment
        
        /* 
         * Format of the "while" statement is:
         *      initialize;
         *      while (test for run condition) {
         *          body  // make progress towards termination condition!
         *      }
         * The "initialize" is done once, before the body executes at all.
         * The "test for run condition" is done before the body executes
         * each loop.  If true, the body is executed.  If false, the "while"
         * statement is complete and so the next statement after the body
         * is executed.
         * The body of the loop executes repeatedly as long as the "run
         * condition" is still true. The body should be doing something
         * that is getting us closer to finishing, or it should know that
         * some outside influence will do something that will eventually
         * cause the run condition to change to false.
         */
        periods = 0;
        toDate = base;
        while (toDate < goal) {
            toDate = toDate+(toDate*rate);
            periods = periods+1;
        }
        System.out.println("\nIt took "+periods+" periods to reach our goal of "+
            goal+",");
        System.out.println("starting at "+base+" and increasing at "+rate+ " per period.");
        
        /*
         * Some common expressions can be shortened
         */
        periods = 0;
        toDate = base;
        while (toDate < goal) {
            toDate += toDate*rate;      // += -> add the right to the left and store in left
            periods++;                  // ++ -> increment the variable
        }
        System.out.println("\nIt took "+periods+" periods to reach our goal of "+
            goal+",");
        System.out.println("starting at "+base+" and increasing at "+rate+ " per period.");
    }
    /**
     * test harness for this class
     * @param arg ignored
     */
    public static void main(String[] arg) {
        Looper dizzy = new Looper();
        dizzy.demoFor(8);
        dizzy.demoWhile(200);
    }
    
    //--------------------------------------------------
    
    /** initial amount for growth calculations */
    double base;
    
    /** fractional growth per period */
    double rate;

}
